@@ -22,6 +22,7 @@
 #include <libaudgui/libaudgui.h>
 
 #include "aud-dbus.h"
+#include "debug.h"
 #include "drct.h"
 #include "main.h"
 #include "misc.h"
@@ -739,18 +740,37 @@ handlers[] =
     {"handle-volume", (GCallback) do_volume}
 };
 
+static GMainLoop * mainloop = NULL;
+static unsigned owner_id = 0;
+
 static GDBusInterfaceSkeleton * skeleton = NULL;
 
-void dbus_server_init (void)
+static void name_acquired (GDBusConnection * conn, const char * name, void * unused)
+{
+    AUDDBG ("Owned D-Bus name (org.atheme.audacious) on session bus.\n");
+
+    g_main_loop_quit (mainloop);
+}
+
+static void name_lost (GDBusConnection * conn, const char * name, void * unused)
+{
+    AUDDBG ("Owning D-Bus name (org.atheme.audacious) failed, already taken?\n");
+
+    g_bus_unown_name (owner_id);
+    owner_id = 0;
+
+    g_main_loop_quit (mainloop);
+}
+
+int dbus_server_init (void)
 {
     GError * error = NULL;
     GDBusConnection * bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, & error);
+    GMainContext * context;
 
     if (! bus)
         goto ERROR;
 
-    g_bus_own_name_on_connection (bus, "org.atheme.audacious", 0, NULL, NULL, NULL, NULL);
-
     skeleton = (GDBusInterfaceSkeleton *) obj_audacious_skeleton_new ();
 
     for (int i = 0; i < ARRAY_LEN (handlers); i ++)
@@ -759,7 +779,25 @@ void dbus_server_init (void)
     if (! g_dbus_interface_skeleton_export (skeleton, bus, "/org/atheme/audacious", & error))
         goto ERROR;
 
-    return;
+    context = g_main_context_new ();
+    g_main_context_push_thread_default (context);
+
+    owner_id = g_bus_own_name (G_BUS_TYPE_SESSION, "org.atheme.audacious", 0,
+     NULL, name_acquired, name_lost, NULL, NULL);
+
+    mainloop = g_main_loop_new (context, TRUE);
+    g_main_loop_run (mainloop);
+    g_main_loop_unref (mainloop);
+    mainloop = NULL;
+
+    g_main_context_pop_thread_default (context);
+    g_main_context_unref (context);
+
+    if (owner_id)
+        return STARTUP_TYPE_SERVER;
+
+    dbus_server_cleanup ();
+    return STARTUP_TYPE_CLIENT;
 
 ERROR:
     if (error)
@@ -767,10 +805,19 @@ ERROR:
         fprintf (stderr, "D-Bus error: %s\n", error->message);
         g_error_free (error);
     }
+
+    dbus_server_cleanup ();
+    return STARTUP_TYPE_UNKNOWN;
 }
 
 void dbus_server_cleanup (void)
 {
+    if (owner_id)
+    {
+        g_bus_unown_name (owner_id);
+        owner_id = 0;
+    }
+
     if (skeleton)
     {
         g_object_unref (skeleton);
