@@ -333,6 +333,10 @@ static void do_remote (void)
     ObjAudacious * obj = NULL;
     GError * error = NULL;
 
+    /* check whether this is the first instance */
+    if (dbus_server_init () != STARTUP_TYPE_CLIENT)
+        return;
+
     if (! (bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, & error)))
         goto ERR;
 
@@ -340,14 +344,7 @@ static void do_remote (void)
      "/org/atheme/audacious", NULL, & error)))
         goto ERR;
 
-    /* check whether remote is running */
-    char * version = NULL;
-    obj_audacious_call_version_sync (obj, & version, NULL, NULL);
-
-    if (! version)
-        goto DONE;
-
-    AUDDBG ("Connected to remote version %s.\n", version);
+    AUDDBG ("Connected to remote session.\n");
 
     /* if no command line options, then present running instance */
     if (! (filenames || options.play || options.pause || options.play_pause ||
@@ -392,20 +389,16 @@ static void do_remote (void)
     if (options.mainwin)
         obj_audacious_call_show_main_win_sync (obj, TRUE, NULL, NULL);
 
-    g_free (version);
     g_object_unref (obj);
 
     exit (EXIT_SUCCESS);
 
 ERR:
-    fprintf (stderr, "D-Bus error: %s\n", error->message);
-    g_error_free (error);
-
-DONE:
-    if (obj)
-        g_object_unref (obj);
-
-    return;
+    if (error)
+    {
+        fprintf (stderr, "D-Bus error: %s\n", error->message);
+        g_error_free (error);
+    }
 }
 #endif
 
@@ -517,10 +510,6 @@ static void init_two (void)
 
     AUDDBG ("Loading highlevel plugins.\n");
     start_plugins_two ();
-
-#ifdef USE_DBUS
-    dbus_server_init ();
-#endif
 }
 
 static void shut_down (void)
