@@ -48,10 +48,7 @@ struct _VFSFile {
 };
 
 /* Audacious core provides us with a function that looks up a VFS transport for
- * a given URI scheme.  Since this function will load plugins as needed, it can
- * only be called from the main thread.  When VFS is used from parallel threads,
- * vfs_prepare must be called from the main thread to look up any needed
- * transports beforehand. */
+ * a given URI scheme.  This function will load plugins as needed. */
 
 static VFSConstructor * (* lookup_func) (const char * scheme) = NULL;
 
@@ -404,7 +401,12 @@ vfs_file_test(const char * path, int test)
     if (strncmp (path, "file://", 7))
         return FALSE; /* only local files are handled */
 
-    char * path2 = uri_to_filename (path);
+    const char * sub;
+    uri_parse (path, NULL, NULL, & sub, NULL);
+
+    SNCOPY (no_sub, path, sub - path);
+
+    char * path2 = uri_to_filename (no_sub);
     if (! path2)
         return FALSE;
 
