@@ -103,7 +103,7 @@ static int64_t local_fread (void * ptr, int64_t size, int64_t nitems, VFSFile *
 
     if (local->last_op == OP_WRITE)
     {
-        if (fseeko (local->stream, 0, SEEK_CUR) < 0)  /* flush buffers */
+        if (fflush (local->stream) < 0)
         {
             perror (local->path);
             return 0;
@@ -127,7 +127,7 @@ static int64_t local_fwrite (const void * ptr, int64_t size, int64_t nitems, VFS
 
     if (local->last_op == OP_READ)
     {
-        if (fseeko (local->stream, 0, SEEK_CUR) < 0)  /* flush buffers */
+        if (fflush (local->stream) < 0)
         {
             perror (local->path);
             return 0;
@@ -178,10 +178,10 @@ static int local_ftruncate (VFSFile * file, int64_t length)
 
     if (local->last_op != OP_NONE)
     {
-        if (fseeko (local->stream, 0, SEEK_CUR) < 0)  /* flush buffers */
+        if (fflush (local->stream) < 0)
         {
             perror (local->path);
-            return 0;
+            return -1;
         }
     }
 
@@ -205,7 +205,7 @@ static int64_t local_fsize (VFSFile * file)
     if (local->cached_size < 0)
     {
         int64_t saved_pos = ftello (local->stream);
-        if (ftello < 0)
+        if (saved_pos < 0)
             goto ERR;
 
         if (local_fseek (file, 0, SEEK_END) < 0)
