@@ -18,17 +18,13 @@
  */
 
 #ifndef TAGUTIL_H
-
 #define TAGUTIL_H
 
 #include <stdint.h>
-#include <time.h>
+#include <stdio.h>
 
-#include "libaudcore/tuple.h"
 #include "libaudcore/vfs.h"
 
-#define BROKEN 1
-
 enum {
     GENRE_BLUES = 0,
     GENRE_CLASSIC_ROCK,
@@ -161,36 +157,18 @@ extern bool_t tag_verbose;
 
 #define TAGDBG(...) do {if (tag_verbose) {printf ("%s:%d [%s]: ", __FILE__, __LINE__, __FUNCTION__); printf (__VA_ARGS__);}} while (0)
 
-time_t unix_time(uint64_t win_time);
-
-uint16_t get_year(uint64_t win_time);
-
-char *read_char_data(VFSFile *fd, int size);
-bool_t write_char_data(VFSFile *f, char *data, size_t i);
-
-uint8_t read_uint8(VFSFile *fd);
-uint16_t read_LEuint16(VFSFile *fd);
-uint16_t read_BEuint16(VFSFile *fd);
-uint32_t read_LEuint32(VFSFile *fd);
-uint32_t read_BEuint32(VFSFile *fd);
-uint64_t read_LEuint64(VFSFile *fd);
-uint64_t read_BEuint64(VFSFile *fd);
-
-
-bool_t write_uint8(VFSFile *fd, uint8_t val);
-bool_t write_BEuint16(VFSFile *fd, uint16_t val);
-bool_t write_LEuint16(VFSFile *fd, uint16_t val);
-bool_t write_BEuint32(VFSFile *fd, uint32_t val);
-bool_t write_LEuint32(VFSFile *fd, uint32_t val);
-bool_t write_BEuint64(VFSFile *fd, uint64_t val);
-bool_t write_LEuint64(VFSFile *fd, uint64_t val);
-
-uint64_t read_LEint64(VFSFile *fd);
-bool_t cut_beginning_tag (VFSFile * handle, int64_t tag_size);
-
 const char *convert_numericgenre_to_text(int numericgenre);
 
 uint32_t unsyncsafe32 (uint32_t x);
 uint32_t syncsafe32 (uint32_t x);
 
-#endif
+typedef struct {
+    char * name;
+    int fd;
+} TempFile;
+
+bool_t open_temp_file_for (TempFile * temp, VFSFile * file);
+bool_t copy_region_to_temp_file (TempFile * temp, VFSFile * file, int64_t offset, int64_t size);
+bool_t replace_with_temp_file (TempFile * temp, VFSFile * file);
+
+#endif /* TAGUTIL_H */
